from __future__ import print_function
from numpy import int16
import time, struct


def connect(route, **args):
    return VL53L01X(route, **args)

def makeuint16(lsb, msb):
	return ((msb & 0xFF) << 8)  | (lsb & 0xFF)


class VL53L01X:
    VL53L0X_REG_IDENTIFICATION_MODEL_ID = 0x00c0
    VL53L0X_REG_IDENTIFICATION_REVISION_ID = 0x00c2
    VL53L0X_REG_PRE_RANGE_CONFIG_VCSEL_PERIOD = 0x0050
    VL53L0X_REG_FINAL_RANGE_CONFIG_VCSEL_PERIOD = 0x0070
    VL53L0X_REG_SYSRANGE_START = 0x000

    VL53L0X_REG_RESULT_INTERRUPT_STATUS = 0x0013
    VL53L0X_REG_RESULT_RANGE_STATUS = 0x0014

    VL53L0X_address = 0x29  # 41

    def __init__(self, I2C):
        self.I2CWriteBulk = I2C.writeBulk
        self.I2CReadBulk = I2C.readBulk
        val1 = self.I2CReadBulk(self.VL53L0X_address, self.VL53L0X_REG_IDENTIFICATION_REVISION_ID, 1)[0]
        print ("Revision ID: " + hex(val1))
        val1 = self.I2CReadBulk(self.VL53L0X_address, self.VL53L0X_REG_IDENTIFICATION_MODEL_ID, 1)[0]
        print ("Device ID: " + hex(val1))
        val1 = self.I2CReadBulk(self.VL53L0X_address, self.VL53L0X_REG_PRE_RANGE_CONFIG_VCSEL_PERIOD, 1)[0]
        print ("PRE_RANGE_CONFIG_VCSEL_PERIOD=" + hex(val1) + " decode: " + str(self.VL53L0X_decode_vcsel_period(val1)))
        val1 = self.I2CReadBulk(self.VL53L0X_address, self.VL53L0X_REG_FINAL_RANGE_CONFIG_VCSEL_PERIOD, 1)[0]

        # print ("FINAL_RANGE_CONFIG_VCSEL_PERIOD=" + hex(val1) + " decode: " + str(self.VL53L0X_decode_vcsel_period(val1)))

    def VL53L0X_decode_vcsel_period(self, vcsel_period_reg):
        vcsel_period_pclks = (vcsel_period_reg + 1) << 1;
        return vcsel_period_pclks


    def getVals(self):
        val1 = self.I2CWriteBulk(self.VL53L0X_address, [self.VL53L0X_REG_SYSRANGE_START, 0x01])
        cnt = 0
        while (cnt < 100):  # 1 second waiting time max
            time.sleep(0.010)
            val = self.I2CReadBulk(self.VL53L0X_address, self.VL53L0X_REG_RESULT_RANGE_STATUS, 1)[0]
            if (val & 0x01):
                break
            cnt += 1
        if (cnt == 100):  # timeout
            return None
        if not (val & 0x01):  # Not ready.
            return None

        #	Status = VL53L0X_ReadMulti(Dev, 0x14, localBuffer, 12);
        data = self.I2CReadBulk(self.VL53L0X_address, 0x14, 12)
        # print ("ambient count " + str(makeuint16(data[7], data[6])))
        # print ("signal count " + str(makeuint16(data[9], data[8])))
        d = makeuint16(data[11], data[10])

        DeviceRangeStatusInternal = ((data[0] & 0x78) >> 3)
        # print (DeviceRangeStatusInternal)

        return [d]



import time,sys
import numpy as np

class AD9833:
    if sys.version.major==3:
        DDS_MAX_FREQ = 0xFFFFFFF-1    #24 bit resolution
    else:
        DDS_MAX_FREQ = eval("0xFFFFFFFL-1")    #24 bit resolution
    #control bytes
    DDS_B28 = 13
    DDS_HLB = 12
    DDS_FSELECT = 11
    DDS_PSELECT = 10
    DDS_RESET = 8
    DDS_SLEEP1 =7
    DDS_SLEEP12 =6
    DDS_OPBITEN =5
    DDS_DIV2 =3
    DDS_MODE =1

    DDS_FSYNC =9

    DDS_SINE =(0)
    DDS_TRIANGLE =(1<<DDS_MODE)
    DDS_SQUARE =(1<<DDS_OPBITEN)
    DDS_RESERVED =(1<<DDS_OPBITEN)|(1<<DDS_MODE)
    clockScaler = 4 # 8MHz
    def __init__(self,I=None):
        self.CS=9
        if I:
            self.I = I
        else:
            from SEEL import interface
            self.I = interface.connect()
        self.I.SPI.set_parameters(2,2,1,1,0)
        self.I.map_reference_clock(self.clockScaler,'WAVEGEN')
        print ('clock set to ',self.I.DDS_CLOCK )

        self.waveform_mode = self.DDS_TRIANGLE;
        self.write(1<<self.DDS_RESET)
        self.write((1<<self.DDS_B28) | self.waveform_mode )               #finished loading data
        self.active_channel = 0
        self.frequency  =  1000

    def write(self,con):
        self.I.SPI.start(self.CS)
        self.I.SPI.send16(con)
        self.I.SPI.stop(self.CS)

    def set_frequency(self,freq,register=0,**args):
        self.active_channel = register
        self.frequency  =  freq

        freq_setting = int(round(freq * self.DDS_MAX_FREQ / self.I.DDS_CLOCK))
        modebits = (1<<self.DDS_B28)| self.waveform_mode
        if register:
            modebits|=(1<<self.DDS_FSELECT)
            regsel = 0x8000
        else:
            regsel=0x4000

        self.write( (1<<self.DDS_RESET) | modebits ) #Ready to load DATA
        self.write( (regsel |  (freq_setting&0x3FFF))&0xFFFF      )           #LSB
        self.write( (regsel | ((freq_setting>>14)&0x3FFF))&0xFFFF )           #MSB
        phase = args.get('phase',0)
        self.write( 0xc000|phase)                            #Phase
        self.write(modebits)               #finished loading data

    def set_voltage(self,v):
        self.waveform_mode=self.DDS_TRIANGLE
        self.set_frequency(0,0,phase = v)#0xfff*v/.6)

    def select_frequency_register(self,register):
        self.active_channel = register
        modebits = self.waveform_mode
        if register:    modebits|=(1<<self.DDS_FSELECT)
        self.write(modebits )


    def set_waveform_mode(self,mode):
        self.waveform_mode=mode
        modebits = mode
        if self.active_channel:    modebits|=(1<<self.DDS_FSELECT)
        self.write(modebits )



if __name__ == "__main__":
    from expeyes import eyes17
    I= eyes17.open()
    A = AD9833(I=I)
    A.set_waveform_mode(A.DDS_SINE)
    A.set_frequency(3600,0)
    A.set_frequency(3600,1)




# -*- coding: utf-8; mode: python; indent-tabs-mode: t; tab-width:4 -*-
'''
Adapted from https://github.com/janheise/TSL2561
'''

from __future__ import print_function
import time

def connect(route,**args):
	return TSL2561(route,**args)

class TSL2561: 
	VISIBLE                   = 2       # channel 0 - channel 1
	INFRARED                  = 1       # channel 1
	FULLSPECTRUM              = 0       # channel 0

	READBIT                   = 0x01
	COMMAND_BIT               = 0x80    # Must be 1

	CONTROL_POWERON           = 0x03
	CONTROL_POWEROFF          = 0x00


	REGISTER_CONTROL          = 0x00
	REGISTER_TIMING           = 0x01
	REGISTER_ID               = 0x0A


	INTEGRATIONTIME_13MS      = 0x00    # 13.7ms
	INTEGRATIONTIME_101MS     = 0x01    # 101ms
	INTEGRATIONTIME_402MS     = 0x02    # 402ms

	GAIN_1X                   = 0x00    # No gain
	GAIN_16X                  = 0x10    # 16x gain

	ADDRESS = 0x39 #addr normal
	timing = INTEGRATIONTIME_13MS
	gain = GAIN_16X
	name = 'TSL2561 Luminosity'
	ADDRESS = 0x39
	NUMPLOTS=3
	PLOTNAMES = ['Full','IR','Visible']
	def __init__(self, I2C,**args):
		self.ADDRESS = args.get('address',0x39)
		self.I2C = I2C  
		# set timing 101ms & 16x gain
		self.initialize(None)

		self.params={'setGain':['1x','16x'],'setTiming':[0,1,2],'initialize':['Init']}

	def initialize(self,v):
		self.enable()
		self.wait()
		self.I2C.writeBulk(self.ADDRESS,[0x80 | 0x01, 0x01 | 0x10 ])
		# full scale luminosity
		infra = self.I2C.readBulk(self.ADDRESS,0x80 | 0x20 | 0x0E ,2)
		full = self.I2C.readBulk(self.ADDRESS,0x80 | 0x20 | 0x0C ,2)
		full = (full[1]<<8)|full[0]
		infra = (infra[1]<<8)|infra[0]

		print("Full:     %04x" % full)
		print("Infrared: %04x" % infra)
		print("Visible:  %04x" % (full - infra) )

		#self.I2C.writeBulk(self.ADDRESS,[0x80,0x00])
		
	def getID(self):
		ID=self.I2C.readBulk(self.ADDRESS,self.REGISTER_ID ,1)
		print (hex(ID))
		return ID
	
	def getRaw(self):
		vals = self.I2C.readBulk(self.ADDRESS,0x80 | 0x20 | 0x0C ,4)  # Byte 1, 2 (full range ) . Byte (3,4) Infrared .
		if vals:
			full = (vals[1]<<8)|vals[0]
			infra = (vals[3]<<8)|vals[2]
			return [full,infra,full-infra]
		else:
			return False

	def setGain(self, gain):
		if(gain=='1x'):self.gain = self.GAIN_1X
		elif(gain=='16x'):self.gain = self.GAIN_16X
		else: self.gain = self.GAIN_0X

		self.I2C.writeBulk(self.ADDRESS,[self.COMMAND_BIT | self.REGISTER_TIMING, self.gain | self.timing])
	
	def setTiming(self, timing):
		print ([13,101,402][timing],'mS')
		self.timing = timing    
		self.I2C.writeBulk(self.ADDRESS,[self.COMMAND_BIT | self.REGISTER_TIMING, self.gain | self.timing])

	def enable(self):
		self.I2C.writeBulk(self.ADDRESS,[self.COMMAND_BIT | self.REGISTER_CONTROL, self.CONTROL_POWERON])

	def disable(self):
		self.I2C.writeBulk(self.ADDRESS,[self.COMMAND_BIT | self.REGISTER_CONTROL, self.CONTROL_POWEROFF])

	def wait(self):
		if self.timing == self.INTEGRATIONTIME_13MS:
			time.sleep(0.14)
		if self.timing == self.INTEGRATIONTIME_101MS:
			time.sleep(0.102)
		if self.timing == self.INTEGRATIONTIME_402MS:
			time.sleep(0.403)        




'''
expEYES data analysis library using numpy and scipy
Author  : Ajith Kumar B.P, bpajith@gmail.com
License : GNU GPL version 3
'''

import sys, time, math
from numpy import *
import numpy.fft
from scipy import optimize 
from scipy.optimize import leastsq
import scipy.optimize as optimize


def find_peak(va):
	vmax = 0.0
	size = len(va)
	index = 0
	for i in range(1,size):		# skip first 2 channels, DC
		if va[i] > vmax:
			vmax = va[i]
			index = i
	return index

#-------------------------- Fourier Transform ------------------------------------
def fft(ya, si):
	'''
	Returns positive half of the Fourier transform of the signal ya. 
	Sampling interval 'si', in milliseconds
	'''
	np = len(ya)
	v = array(ya)
	tr = abs(numpy.fft.fft(v))/np
	frq = numpy.fft.fftfreq(np, si * 1.0e-3)
        # the arguments for reshape must be integers: enforcing
        # the integer division by two.
	x = frq.reshape(2,np//2)
	y = tr.reshape(2,np//2)
	return x[0], y[0]    

def find_frequency(x,y):		# Returns the fundamental frequency using FFT
	tx,ty = fft(y, x[1]-x[0])
	index = find_peak(ty)
	if index == 0:
		return None
	else:
		return tx[index]

#-------------------------- Sine Fit ------------------------------------------------
def sine_erf(p,y,x):					
	return y - p[0] * sin(2*pi*p[1]*x+p[2])+p[3]

def sine_eval(x,p):			# y = a * sin(2*pi*f*x + phi)+ offset
	return p[0] * sin(2*pi*p[1]*x+p[2])-p[3]

def sineFunc(x, a1, a2, a3, a4):
    return a4 + a1*sin(abs(a2*(2*pi))*x + a3)

def fit_sine(xa,ya, freq = 0):	# Time in mS, V in volts, freq in Hz, accepts numpy arrays
	size = len(ya)
	mx = max(ya)
	mn = min(ya)
	amp = (mx-mn)/2
	if freq == 0:						# Guess frequency not given
		freq = find_frequency(xa,ya)
	if freq == None:
		return None
	#print 'guess a & freq = ', amp, freq
	par = [abs(amp), freq*0.001, 0.0, 0.0] # Amp, freq, phase , offset
	par, pcov = optimize.curve_fit(sineFunc, xa, ya, par)
	if par[0] < 0:		# Negative amplitude ?
		par[0] *= -1
		par[2] += pi
	yfit = sine_eval(xa, par)
	return yfit,par
	

#--------------------------Damped Sine Fit ------------------------------------------------
def dsine_erf(p,y,x):
	return y - p[0] * sin(2*pi*p[1]*x+p[2]) * exp(-p[4]*x) + p[3]

def dsine_eval(x,p):
	return     p[0] * sin(2*pi*p[1]*x+p[2]) * exp(-p[4]*x) - p[3]

def fit_dsine(xlist, ylist, freq = 0):
	size = len(xlist)
	xa = array(xlist, dtype=float)
	ya = array(ylist, dtype=float)
	amp = (max(ya)-min(ya))/2
	if freq == 0:
		freq = find_frequency(xa,ya)
	par = [amp, freq*0.001, 0.0, 0.0, 0.1] # Amp, freq, phase , offset, decay constant
	plsq = leastsq(dsine_erf, par,args=(ya,xa))
	if plsq[1] > 4:
		return None
	yfit = dsine_eval(xa, plsq[0])
	return yfit,plsq[0]

#-------------------------- Exponential Fit ----------------------------------------
def exp_erf(p,y,x):
	return y - p[0] * exp(p[1]*x) + p[2]

def exp_eval(x,p):
	return p[0] * exp(p[1]*x)  -p[2]

def fit_exp(xlist, ylist):
	size = len(xlist)
	xa = array(xlist, dtype=float)
	ya = array(ylist, dtype=float)
	maxy = max(ya)
	halfmaxy = maxy / 2.0
	halftime = 1.0
	for k in range(size):
		if abs(ya[k] - halfmaxy) < halfmaxy/100:
			halftime = xa[k]
			break 
	par = [maxy, -halftime,0] 					# Amp, decay, offset
	plsq = leastsq(exp_erf, par,args=(ya,xa))
	if plsq[1] > 4:
		return None
	yfit = exp_eval(xa, plsq[0])
	return yfit,plsq[0]

	#-------------------------- Exponential Fit #2----------------------------------------

def exp_func(x, a, b, c):
	return a * exp(-x/ b) + c

def fit_exp2(t,v):    # accepts numpy arrays
	size = len(t)
	v80 = v[0] * 0.8
	for k in range(size-1):
		if v[k] < v80:
			rc = t[k]/.223
			break
	pg = [v[0], rc, 0]
	po, err = optimize.curve_fit(exp_func, t, v, pg)
	if abs(err[0][0]) > 0.1:
		return None, None
	vf = po[0] * exp(-t/po[1]) + po[2]
	return po, vf	


#-------------------------- Gauss Fit ----------------------------------------
def gauss_erf(p,y,x):#height, mean, sigma
	return y - p[0] * exp(-(x-p[1])**2 /(2.0 * p[2]**2))

def gauss_eval(x,p):
	return p[0] * exp(-(x-p[1])**2 /(2.0 * p[2]**2))

def fit_gauss(xlist, ylist):
	size = len(xlist)
	xa = array(xlist, dtype=float)
	ya = array(ylist, dtype=float) 
	maxy = max(ya)
	halfmaxy = maxy / 2.0
	for k in range(size):
		if abs(ya[k] - maxy) < maxy/100:
			mean = xa[k]
			break
	for k in range(size):
		if abs(ya[k] - halfmaxy) < halfmaxy/10:
			halfmaxima = xa[k]
			break                      
	sigma = mean - halfmaxima
	par = [maxy, halfmaxima, sigma] # Amplitude, mean, sigma
	plsq = leastsq(gauss_erf, par,args=(ya,xa))
	if plsq[1] > 4:
		return None
	yfit = gauss_eval(xa, plsq[0])
	return yfit,plsq[0]

#-------------------------- liniar Fit ------------------------------------------------
def line_erf(p,y,x):					
	return y - p[0] * x - p[1]

def line_eval(x,p):			# y = a * x + b
	return p[0] * x + p[1]

def fit_line(xlist,ylist):	# Time in mS, V in volts
	size = len(xlist)
	xa = array(xlist, dtype=float)
	ya = array(ylist, dtype=float)
	par = [1,1] # m, c
	plsq = leastsq(line_erf, par,args=(ya,xa))
	if plsq[1] > 4:
		return None
	yfit = line_eval(xa, plsq[0])
	return yfit,plsq[0]

#-------------------------- Quadratic Fit ----------------------------------------
def qdr_erf(p,y,x):
	return y - (p[0] * x**2 +p[1]*x + p[2]) # ax^2 + bx + c

def qdr_eval(x,p):
	return p[0] * x**2 +p[1]*x + p[2]

def fit_qdr(xlist, ylist):
	size = len(xlist)
	xa = array(xlist, dtype=float)
	ya = array(ylist, dtype=float)
	par = [1, 1, 1] 					# a,b,c
	plsq = leastsq(qdr_erf, par,args=(ya,xa))
	if plsq[1] > 4:
		return None
	yfit = qdr_eval(xa, plsq[0])
	return yfit,plsq[0]




# -*- coding: utf-8; mode: python; indent-tabs-mode: t; tab-width:4 -*-
from __future__ import print_function
from . import commands_proto as CP
import numpy as np
import time, inspect


class I2C():
    """
	Methods to interact with the I2C port.

	.. code-block:: python

		#Code Example : Read Values from an HMC5883L 3-axis Magnetometer(compass) [GY-273 sensor] connected to the I2C port
		ADDRESS = 0x1E
		import expeyes.eyes17
		I = expeyes.eyes17.connect() 
		
		# writing to 0x1E, set gain(0x01) to smallest(0 : 1x)
		I.I2C.bulkWrite(ADDRESS,[0x01,0])
		
		# writing to 0x1E, set mode conf(0x02), continuous measurement(0)
		I.I2C.bulkWrite(ADDRESS,[0x02,0])

		# read 6 bytes from addr register on I2C device located at ADDRESS
		vals = I.I2C.bulkRead(ADDRESS,addr,6)
			
		from numpy import int16
		#conversion to signed datatype
		x=int16((vals[0]<<8)|vals[1])
		y=int16((vals[2]<<8)|vals[3])
		z=int16((vals[4]<<8)|vals[5])
		print (x,y,z)

	"""

    def __init__(self, H):
        self.H = H
        from . import sensorlist
        self.SENSORS = sensorlist.sensors
        self.buff = np.zeros(10000)

    def init(self):
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_INIT)
            self.H.__get_ack__()
        except Exception as ex:
            print(ex, "Communication Error")
            return False

    def enable_smbus(self):
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_ENABLE_SMBUS)
            self.H.__get_ack__()
        except Exception as ex:
            print(ex, "Communication Error")
            return False

    def pullSCLLow(self, uS):
        """
		Hold SCL pin at 0V for a specified time period. Used by certain sensors such
		as MLX90316 PIR for initializing.
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		uS                  Time(in uS) to hold SCL output at 0 Volts
		================    ============================================================================================

		"""
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_PULLDOWN_SCL)
            self.H.__sendInt__(uS)
            self.H.__get_ack__()
        except Exception as ex:
            print(ex, "Communication Error")
            return False

    def config(self, freq, verbose=True):
        """
		Sets frequency for I2C transactions
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		freq                I2C frequency
		================    ============================================================================================
		"""
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_CONFIG)
            # freq=1/((BRGVAL+1.0)/64e6+1.0/1e7)
            BRGVAL = int((1. / freq - 1. / 1e7) * 64e6 - 1)
            if BRGVAL > 511:
                BRGVAL = 511
                if verbose: print('Frequency too low. Setting to :', 1 / ((BRGVAL + 1.0) / 64e6 + 1.0 / 1e7))
            self.H.__sendInt__(BRGVAL)
            self.H.__get_ack__()
        except Exception as ex:
            print(ex, "Communication Error")
            return False

    def start(self, address, rw):
        """
		Initiates I2C transfer to address via the I2C port
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		address             I2C slave address\n
		rw                  Read/write.
							- 0 for writing
							- 1 for reading.
		================    ============================================================================================
		"""
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_START)
            self.H.__sendByte__(((address << 1) | rw) & 0xFF)  # address
            return self.H.__get_ack__() >> 4
        except Exception as ex:
            print(ex, "Communication Error")
            return False

    def stop(self):
        """
		stops I2C transfer
		
		:return: Nothing
		"""
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_STOP)
            self.H.__get_ack__()
        except Exception as ex:
            print(ex, "Communication Error")
            return False

    def wait(self):
        """
		wait for I2C

		:return: Nothing
		"""
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_WAIT)
            self.H.__get_ack__()
        except Exception as ex:
            print(ex, "Communication Error")
            return False

    def send(self, data):
        """
		SENDS data over I2C.
		The I2C bus needs to be initialized and set to the correct slave address first.
		Use I2C.start(address) for this.
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		data                Sends data byte over I2C bus
		================    ============================================================================================

		:return: Nothing
		"""
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_SEND)
            self.H.__sendByte__(data)  # data byte
            return self.H.__get_ack__() >> 4
        except Exception as ex:
            print(ex, "Communication Error")
            return False

    def send_burst(self, data):
        """
		SENDS data over I2C. The function does not wait for the I2C to finish before returning.
		It is used for sending large packets quickly.
		The I2C bus needs to be initialized and set to the correct slave address first.
		Use start(address) for this.

		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		data                Sends data byte over I2C bus
		================    ============================================================================================

		:return: Nothing
		"""
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_SEND_BURST)
            self.H.__sendByte__(data)  # data byte
        except Exception as ex:
            print(ex, "Communication Error")
            return False

    def restart(self, address, rw):
        """
		Initiates I2C transfer to address

		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		address             I2C slave address
		rw                  Read/write.
							* 0 for writing
							* 1 for reading.
		================    ============================================================================================

		"""
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_RESTART)
            self.H.__sendByte__(((address << 1) | rw) & 0xFF)  # address
        except Exception as ex:
            print(ex, "Communication Error")
        return self.H.__get_ack__() >> 4

    def simpleRead(self, addr, numbytes):
        """
		Read bytes from I2C slave without first transmitting the read location.
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		addr                Address of I2C slave
		numbytes            Total Bytes to read
		================    ============================================================================================
		"""
        self.start(addr, 1)
        vals = self.read(numbytes)
        return vals

    def read(self, length):
        """
		Reads a fixed number of data bytes from I2C device. Fetches length-1 bytes with acknowledge bits for each, +1 byte
		with Nack.

		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		length              number of bytes to read from I2C bus
		================    ============================================================================================
		"""
        data = []
        try:
            for a in range(length - 1):
                self.H.__sendByte__(CP.I2C_HEADER)
                self.H.__sendByte__(CP.I2C_READ_MORE)
                data.append(self.H.__getByte__())
                self.H.__get_ack__()
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_READ_END)
            data.append(self.H.__getByte__())
            self.H.__get_ack__()
        except Exception as ex:
            print(ex, "Communication Error")
            return False
        return data

    def read_repeat(self):
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_READ_MORE)
            val = self.H.__getByte__()
            self.H.__get_ack__()
        except Exception as ex:
            print(ex, "Communication Error")
            return False
        return val

    def read_end(self):
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_READ_END)
            val = self.H.__getByte__()
            self.H.__get_ack__()
        except Exception as ex:
            print(ex, "Communication Error")
            return False
        return val

    def read_status(self):
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_STATUS)
            val = self.H.__getInt__()
            self.H.__get_ack__()
        except Exception as ex:
            print(ex, "Communication Error")
            return False
        return val

    def readBulk(self, device_address, register_address, bytes_to_read):
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_READ_BULK)
            self.H.__sendByte__(device_address)
            self.H.__sendByte__(register_address)
            self.H.__sendByte__(bytes_to_read)
            data = self.H.fd.read(bytes_to_read)
            self.H.__get_ack__()
            try:
                # print (data)
                # The following try block should be resolved depending on Python Version. P3-serial treats it as Byte arrays
                try:
                    return [int(a) for a in data]
                except:
                    return [ord(a) for a in data]

            except Exception as e:
                print('Transaction failed', str(e))
                return False
        except Exception as ex:
            print(ex, "Communication Error")
            return False

    def writeBulk(self, device_address, bytestream):
        """
		write bytes to I2C slave
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		device_address      Address of I2C slave
		bytestream          List of bytes to write
		================    ============================================================================================
		"""
        try:
            self.H.__sendByte__(CP.I2C_HEADER)
            self.H.__sendByte__(CP.I2C_WRITE_BULK)
            self.H.__sendByte__(device_address)
            self.H.__sendByte__(len(bytestream))
            for a in bytestream:
                self.H.__sendByte__(a)
            self.H.__get_ack__()
        except Exception as ex:
            print(ex, "Communication Error")
            return False

    def scan(self, frequency=200000, verbose=False):
        """
		Scan I2C port for connected devices
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		Frequency           I2C clock frequency
		================    ============================================================================================

		:return: Array of addresses of connected I2C slave devices

		"""

        self.config(frequency, verbose)
        addrs = []
        n = 0
        if verbose:
            print('Scanning addresses 0-127...')
            print('Address', '\t', 'Possible Devices')
        self.stop()
        for a in range(0, 128):
            x = self.start(a, 0)
            if x & 1 == 0:  # ACK received
                addrs.append(a)
                if verbose: print(hex(a), '\t\t', self.SENSORS.get(a, 'None'))
                n += 1
            self.stop()

        return addrs


class SPI():
    """
	Methods to interact with the SPI port. An instance of Packet_Handler must be passed to the init function
	Available in the eyes20 form factor onwards. version_number >= 5.0:
	"""
    DDS_CLOCK = 25e6

    def __init__(self, H):
        self.H = H

    def set_parameters(self, primary_prescaler=0, secondary_prescaler=2, CKE=1, CKP=0, SMP=1):
        """
		sets SPI parameters.
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		primary_pres        Primary Prescaler(0,1,2,3) for 64MHz clock->(64:1,16:1,4:1,1:1)
		secondary_pres      Secondary prescaler(0,1,..7)->(8:1,7:1,..1:1)
		CKE                 CKE 0 or 1.
		CKP                 CKP 0 or 1.
		================    ============================================================================================
		"""
        try:
            self.H.__sendByte__(CP.SPI_HEADER)
            self.H.__sendByte__(CP.SET_SPI_PARAMETERS)
            # 0Bhgfedcba - > <g>: modebit CKP,<f>: modebit CKE, <ed>:primary pre,<cba>:secondary pre
            self.H.__sendByte__(secondary_prescaler | (primary_prescaler << 3) | (CKE << 5) | (CKP << 6) | (SMP << 7))
            self.H.__get_ack__()
        except Exception as ex:
            self.raiseException(ex, "Communication Error , Function : " + inspect.currentframe().f_code.co_name)

    def start(self, channel):
        """
		selects SPI channel to enable.
		Basically lowers the relevant chip select pin .
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		channel             'CS1','CS2'
		================    ============================================================================================
		
		"""
        self.set_cs(channel, 0)

    def set_cs(self, channel, state):
        """
		Enable or disable a chip select
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		channel             'CS1','CS2'
		state               1 for HIGH, 0 for LOW
		================    ============================================================================================
		
		"""
        try:
            chip_sels = ['NONE', 'A1', 'A2', 'CS1', 'CS2', 'CS3', 'CS4']
            channel = channel.upper()
            if channel in chip_sels:
                csnum = chip_sels.index(channel)
                self.H.__sendByte__(CP.SPI_HEADER)
                if state:
                    self.H.__sendByte__(CP.STOP_SPI)
                else:
                    self.H.__sendByte__(CP.START_SPI)
                self.H.__sendByte__(csnum)
            else:
                print('Channel does not exist')
        except Exception as ex:
            self.raiseException(ex, "Communication Error , Function : " + inspect.currentframe().f_code.co_name)

    def stop(self, channel):
        """
		selects SPI channel to disable.
		Sets the relevant chip select pin to HIGH.
		
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		channel             'CS1','CS2'
		================    ============================================================================================
		
		"""
        self.set_cs(channel, 1)

    def send8(self, value):
        """
		SENDS 8-bit data over SPI
		
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		value               value to transmit
		================    ============================================================================================
		:return: value returned by slave device
		"""
        self.H.__sendByte__(CP.SPI_HEADER)
        self.H.__sendByte__(CP.SEND_SPI8)
        self.H.__sendByte__(value)  # value byte
        v = self.H.__getByte__()
        self.H.__get_ack__()
        return v

    def send16(self, value):
        """
		SENDS 16-bit data over SPI
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		value               value to transmit
		================    ============================================================================================
		:return: value returned by slave device
		:rtype: int
		"""
        self.H.__sendByte__(CP.SPI_HEADER)
        self.H.__sendByte__(CP.SEND_SPI16)
        self.H.__sendInt__(value)  # value byte
        v = self.H.__getInt__()
        self.H.__get_ack__()
        return v

    def send8_burst(self, value):
        """
		SENDS 8-bit data over SPI
		No acknowledge/return value
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		================    ============================================================================================
		**Arguments** 
		================    ============================================================================================
		value               value to transmit
		================    ============================================================================================
		:return: Nothing
		"""
        self.H.__sendByte__(CP.SPI_HEADER)
        self.H.__sendByte__(CP.SEND_SPI8_BURST)
        self.H.__sendByte__(value)  # value byte

    def send16_burst(self, value):
        """
		SENDS 16-bit data over SPI
		no acknowledge/return value
		.. tabularcolumns:: |p{3cm}|p{11cm}|
		
		==============  ============================================================================================
		**Arguments** 
		==============  ============================================================================================
		value           value to transmit
		==============  ============================================================================================
		:return: nothing
		"""
        self.H.__sendByte__(CP.SPI_HEADER)
        self.H.__sendByte__(CP.SEND_SPI16_BURST)
        self.H.__sendInt__(value)  # value byte

    def xfer(self, chan, data):
        self.start(chan)
        reply = []
        for a in data:
            reply.append(self.send8(a))
        self.stop(chan)
        return reply

    def map_reference_clock(self, scaler, *args):
        """
		Map the internal oscillator output  to SQR1,SQR2,SQR3,SQR4 or WAVEGEN
		The output frequency is 128/(1<<scaler) MHz
		
		scaler [0-15]
			
			* 0 -> 128MHz
			* 1 -> 64MHz
			* 2 -> 32MHz
			* 3 -> 16MHz
			* .
			* .
			* 15 ->128./32768 MHz
		example::
		
		>>> I.map_reference_clock(2,'SQR1','SQR2')
		
		outputs 32 MHz on SQR1, SQR2 pins
		
		.. note::
			if you change the reference clock for 'wavegen' , the external waveform generator(AD9833) resolution and range will also change.
			default frequency for 'wavegen' is 16MHz. Setting to 1MHz will give you 16 times better resolution, but a usable range of
			0Hz to about 100KHz instead of the original 2MHz.
		
		"""
        self.H.__sendByte__(CP.WAVEGEN)
        self.H.__sendByte__(CP.MAP_REFERENCE)
        chan = 0
        if 'SQR1' in args: chan |= 1
        if 'SQR2' in args: chan |= 2
        if 'CS3' in args: chan |= 4  # CS3
        if 'WAVEGEN' in args: chan |= 8  # CS3
        self.H.__sendByte__(chan)
        self.H.__sendByte__(scaler)
        if 'WAVEGEN' in args: self.DDS_CLOCK = 128e6 / (1 << scaler)
        self.H.__get_ack__()


class DACCHAN:
    def __init__(self, name, span, channum, **kwargs):
        self.name = name
        self.channum = channum
        self.VREF = kwargs.get('VREF', 0)
        self.SwitchedOff = kwargs.get('STATE', 0)
        self.range = span
        slope = (span[1] - span[0])
        intercept = span[0]
        self.VToCode = np.poly1d([4095. / slope, -4095. * intercept / slope])
        self.CodeToV = np.poly1d([slope / 4095., intercept])
        self.calibration_enabled = False
        self.calibration_table = []
        self.slope = 1
        self.offset = 0

    def load_calibration_twopoint(self, slope, offset):
        self.calibration_enabled = 'twopoint'
        self.slope = slope
        self.offset = offset

    def load_calibration_polynomial(self, poly):
        self.calibration_enabled = 'poly'
        self.polynomial = np.poly1d(poly)

    def apply_calibration(self, v):
        if self.calibration_enabled == 'twopoint':  # Overall slope and offset correction is applied
            return int(np.clip(v * self.slope + self.offset, 0, 4095))
        elif self.calibration_enabled == 'poly':  # 3 degree polynomial
            return int(np.clip(self.polynomial(v), 0, 4095))
        else:
            return v


class PWMDAC:
    defaultVDD = 3300.
    RESET = 6
    WAKEUP = 9
    UPDATE = 8
    WRITEALL = 64
    WRITEONE = 88
    SEQWRITE = 80
    VREFWRITE = 128
    GAINWRITE = 192
    POWERDOWNWRITE = 160
    GENERALCALL = 0

    def __init__(self, H, vref=3.3, devid=0):
        self.devid = devid
        self.addr = 0x60 | self.devid  # 0x60 is the base address
        self.H = H
        self.VREF = vref
        self.I2C = I2C(self.H)
        self.SWITCHEDOFF = [0, 0, 0, 0]
        self.VREFS = [0, 0, 0, 0]  # 0=Vdd,1=Internal reference
        self.CHANS = {'PV2': DACCHAN('PV2', [-3.3, 3.3], 2), 'PV1': DACCHAN('PV1', [-5., 5.], 3)}
        self.CHANNEL_MAP = {0: 'PCS', 2: 'PV2', 3: 'PV1'}
        self.values = {'PV1': 0, 'PV2': 0, 'PCS': 0}

    def __ignoreCalibration__(self, name):
        self.CHANS[name].calibration_enabled = False

    def setVoltage(self, name, v):
        chan = self.CHANS[name]
        v = int(round(chan.VToCode(v)))
        return self.__setRawVoltage__(name, v)

    def getVoltage(self, name):
        return self.values[name]

    def __setRawVoltage__(self, name, v):
        CHAN = self.CHANS[name]
        val = self.CHANS[name].apply_calibration(v)
        self.H.__sendByte__(CP.DAC)  # DAC write coming through.(PWM DAC)
        self.H.__sendByte__(CP.SET_DAC)
        if name == 'PV1':
            self.H.__sendInt__(val)
        else:
            self.H.__sendInt__(0x8000 | val)
        self.H.__get_ack__()

        self.values[name] = CHAN.CodeToV(v)
        return self.values[name]



from numpy import int16
def connect(route,**args):
	return QMC5883L(route,**args)


class QMC5883L():			
	#--------------Parameters--------------------
	#This must be defined in order to let GUIs automatically create menus
	#for changing various options of this sensor
	#It's a dictionary of the string representations of functions matched with an array
	#of options that each one can accept
	params={	'init':['Now'],
	'QMC_RANGE':['2G','8G'],
	}
	ADDRESS = 13
	name = 'Magnetometer'
	NUMPLOTS=3	
	PLOTNAMES = ['Bx','By','Bz']
	QMC_scaling = 3000
	def __init__(self,I2C,**args):
		self.I2C=I2C
		self.ADDRESS = args.get('address',self.ADDRESS)
		self.name = '3 axis Magnetometer'
		self.init('')

	def init(self,dummy_variable_to_circumvent_framework_limitation):  # I know how to fix this now. remind me.
		self.I2C.writeBulk(self.ADDRESS,[0x0A,0x80]) #0x80=reset. 0x40= rollover
		self.I2C.writeBulk(self.ADDRESS,[0x0B,0x01]) #init , set/reset period
		self.QMC_RANGE('2G')

	def QMC_RANGE(self,r):
		if r=='2G':
			self.I2C.writeBulk(self.QMC5883L_ADDRESS,[0x09,0b001|0b000 | 0b100 | 0b10000]) #Mode. continuous|oversampling(512) | rate 50Hz | range(8g)
			self.QMC_scaling = 3000
		elif r=='8G':
			self.I2C.writeBulk(self.QMC5883L_ADDRESS,[0x09,0b001|0b000 | 0b100 | 0b00000]) #Mode. continuous|oversampling(512) | rate 50Hz | range(2g)
			self.QMC_scaling = 12000	

	def getVals(self,addr,numbytes):
		vals = self.I2C.readBulk(self.ADDRESS,addr,numbytes) 
		return vals
	
	def getRaw(self):
		vals=self.getVals(0x00,6)
		if vals:
			if len(vals)==6:
				return [int16((vals[a*2+1]<<8)|vals[a*2])/self.QMC_scaling for a in range(3)]
			else:
				return False
		else:
			return False		

if __name__ == "__main__":
	print('hello')



from __future__ import print_function
from numpy import int16
import time, struct


def connect(route, **args):
    return BMP280(route, **args)


class BMP280:
    ####### BMP280 ###################
    # https://www.bosch-sensortec.com/media/boschsensortec/downloads/datasheets/bst-bme280-ds002.pdf
    ## Partly from https://github.com/farmerkeith/BMP280-library/blob/master/farmerkeith_BMP280.cpp
    BMP280_ADDRESS = 118
    BMP280_REG_CONTROL = 0xF4
    BMP280_REG_RESULT = 0xF6
    BMP280_HUMIDITY_ENABLED = False
    _BMP280_humidity_calib = [0] * 6
    BMP280_oversampling = 0
    _BMP280_PRESSURE_MIN_HPA = 0
    _BMP280_PRESSURE_MAX_HPA = 1600
    _BMP280_sea_level_pressure = 1013.25  # for calibration.. from circuitpython library

    def __init__(self, I2C):
        self.I2CWriteBulk = I2C.writeBulk
        self.I2CReadBulk = I2C.readBulk

        b = self.I2CWriteBulk(self.BMP280_ADDRESS, [0xE0, 0xB6])  # reset
        time.sleep(0.1)
        self.BMP280_HUMIDITY_ENABLED = False
        b = self.I2CReadBulk(self.BMP280_ADDRESS, 0xD0, 1)
        print(b)
        if b is None:
            return
        b = b[0]
        if b in [0x58, 0x56, 0x57]:
            print('BMP280. ID:', b)
        elif b == 0x60:
            self.BMP280_HUMIDITY_ENABLED = True
            print('BME280 . includes humidity')
        else:
            print('ID unknown', b)
        # get calibration data
        b = self.I2CReadBulk(self.BMP280_ADDRESS, 0x88, 24)  # 24 bytes containing calibration data
        coeff = list(struct.unpack('<HhhHhhhhhhhh', bytes(b)))
        coeff = [float(i) for i in coeff]
        self._BMP280_temp_calib = coeff[:3]
        self._BMP280_pressure_calib = coeff[3:]
        self._BMP280_t_fine = 0.

        if self.BMP280_HUMIDITY_ENABLED:
            self.I2CWriteBulk(self.BMP280_ADDRESS, [0xF2, 0b101])  # ctrl_hum. oversampling x 16
            # humidity calibration read
            self._BMP280_humidity_calib = [0] * 6
            self._BMP280_humidity_calib[0] = self.I2CReadBulk(self.BMP280_ADDRESS, 0xA1, 1)[0]  # H1
            coeff = self.I2CReadBulk(self.BMP280_ADDRESS, 0xE1, 7)
            coeff = list(struct.unpack('<hBbBbb', bytes(coeff)))
            self._BMP280_humidity_calib[1] = float(coeff[0])
            self._BMP280_humidity_calib[2] = float(coeff[1])
            self._BMP280_humidity_calib[3] = float((coeff[2] << 4) | (coeff[3] & 0xF))
            self._BMP280_humidity_calib[4] = float((coeff[4] << 4) | (coeff[3] >> 4))
            self._BMP280_humidity_calib[5] = float(coeff[5])

        self.I2CWriteBulk(self.BMP280_ADDRESS, [0xF4, 0xFF])  # ctrl_meas (oversampling of pressure, temperature)

    def _BMP280_calcTemperature(self, adc_t):
        v1 = (adc_t / 16384.0 - self._BMP280_temp_calib[0] / 1024.0) * self._BMP280_temp_calib[1]
        v2 = ((adc_t / 131072.0 - self._BMP280_temp_calib[0] / 8192.0) * (
                adc_t / 131072.0 - self._BMP280_temp_calib[0] / 8192.0)) * self._BMP280_temp_calib[2]
        self._BMP280_t_fine = int(v1 + v2)
        return (v1 + v2) / 5120.0  # actual temperature.

    def _BMP280_calcPressure(self, adc_p, adc_t):
        self._BMP280_calcTemperature(adc_t)  # t_fine has been set now.
        # Algorithm from the BMP280 driver. adapted from adafruit adaptation of
        # https://github.com/BoschSensortec/BMP280_driver/blob/master/bmp280.c
        var1 = self._BMP280_t_fine / 2.0 - 64000.0
        var2 = var1 * var1 * self._BMP280_pressure_calib[5] / 32768.0
        var2 = var2 + var1 * self._BMP280_pressure_calib[4] * 2.0
        var2 = var2 / 4.0 + self._BMP280_pressure_calib[3] * 65536.0
        var3 = self._BMP280_pressure_calib[2] * var1 * var1 / 524288.0
        var1 = (var3 + self._BMP280_pressure_calib[1] * var1) / 524288.0
        var1 = (1.0 + var1 / 32768.0) * self._BMP280_pressure_calib[0]
        if not var1:
            return self._BMP280_PRESSURE_MIN_HPA
        pressure = 1048576.0 - adc_p
        pressure = ((pressure - var2 / 4096.0) * 6250.0) / var1
        var1 = self._BMP280_pressure_calib[8] * pressure * pressure / 2147483648.0
        var2 = pressure * self._BMP280_pressure_calib[7] / 32768.0
        pressure = pressure + (var1 + var2 + self._BMP280_pressure_calib[6]) / 16.0
        pressure /= 100
        if pressure < self._BMP280_PRESSURE_MIN_HPA:
            return self._BMP280_PRESSURE_MIN_HPA
        if pressure > self._BMP280_PRESSURE_MAX_HPA:
            return self._BMP280_PRESSURE_MAX_HPA
        return pressure

    def _BMP280_calcHumidity(self, adc_h, adc_t):
        self._BMP280_calcTemperature(adc_t)  # t fine set.
        var1 = float(self._BMP280_t_fine) - 76800.0
        var2 = (self._BMP280_humidity_calib[3] * 64.0 + (self._BMP280_humidity_calib[4] / 16384.0) * var1)
        var3 = adc_h - var2
        var4 = self._BMP280_humidity_calib[1] / 65536.0
        var5 = (1.0 + (self._BMP280_humidity_calib[2] / 67108864.0) * var1)
        var6 = 1.0 + (self._BMP280_humidity_calib[5] / 67108864.0) * var1 * var5
        var6 = var3 * var4 * (var5 * var6)
        humidity = var6 * (1.0 - self._BMP280_humidity_calib[0] * var6 / 524288.0)
        if humidity > 100:
            return 100
        if humidity < 0:
            return 0

        return humidity

    def getVals(self):
        if self.BMP280_HUMIDITY_ENABLED:
            data = self.I2CReadBulk(self.BMP280_ADDRESS, 0xF7, 8)
        else:
            data = self.I2CReadBulk(self.BMP280_ADDRESS, 0xF7, 6)
        if data is None: return None
        if None not in data:
            # Convert pressure and temperature data to 19-bits
            adc_p = (((data[0] & 0xFF) * 65536.) + ((data[1] & 0xFF) * 256.) + (data[2] & 0xF0)) / 16.
            adc_t = (((data[3] & 0xFF) * 65536.) + ((data[4] & 0xFF) * 256.) + (data[5] & 0xF0)) / 16.
            if self.BMP280_HUMIDITY_ENABLED:
                adc_h = (data[6] * 256.) + data[7]
                return [self._BMP280_calcPressure(adc_p, adc_t), self._BMP280_calcTemperature(adc_t),
                        self._BMP280_calcHumidity(adc_h, adc_t)]
            else:
                return [self._BMP280_calcPressure(adc_p, adc_t), self._BMP280_calcTemperature(adc_t), 0]

        return None

    def readPressure(self):
        x = self.getVals()
        if x:
            return x[0]

    def readTemperature(self):
        x = self.getVals()
        if x:
            return x[1]

    def readHumidity(self):
        x = self.getVals()
        if x:
            return x[2]

    def altitude(self):
        # baseline pressure needs to be provided
        return (44330.0 * (1 - pow(self.P / self.baseline, 1 / 5.255)))

    def sealevel(self, P, A):
        '''
		given a calculated pressure and altitude, return the sealevel
		'''
        return (P / pow(1 - (A / 44330.0), 5.255))



# -*- coding: utf-8; mode: python; indent-tabs-mode: t; tab-width:4 -*-
from numpy import int16,std

import struct

def connect(route,**args):
	return MPU925x(route,**args)

class MPU925x():
	'''
	Mandatory members:
	GetRaw : Function called by Graphical apps. Must return values stored in a list
	NUMPLOTS : length of list returned by GetRaw. Even single datapoints need to be stored in a list before returning
	PLOTNAMES : a list of strings describing each element in the list returned by GetRaw. len(PLOTNAMES) = NUMPLOTS
	name : the name of the sensor shown to the user
	params:
		A dictionary of function calls(single arguments only) paired with list of valid argument values. (Primitive. I know.)
		These calls can be used for one time configuration settings

	'''
	INT_PIN_CFG = 0x37
	GYRO_CONFIG = 0x1B
	ACCEL_CONFIG = 0x1C
	GYRO_SCALING= [131,65.5,32.8,16.4]
	ACCEL_SCALING=[16384,8192,4096,2048]
	AR=3
	GR=3
	NUMPLOTS=7
	PLOTNAMES = ['Ax','Ay','Az','Temp','Gx','Gy','Gz']
	ADDRESS = 0x68
	AK8963_ADDRESS =0x0C
	AK8963_CNTL = 0x0A
	name = 'Accel/gyro'
	def __init__(self,I2C,**args):
		self.I2C=I2C
		self.ADDRESS = args.get('address',self.ADDRESS)
		self.name = 'Accel/gyro'
		self.params={'powerUp':['Go'],'setGyroRange':[250,500,1000,2000],'setAccelRange':[2,4,8,16]}
		self.setGyroRange(2000)
		self.setAccelRange(16)
		'''
		try:
			self.I2C.configI2C(400e3)
		except:
			pass
		'''
		self.powerUp(True)

	def getVals(self,addr,bytes):
		vals = self.I2C.readBulk(self.ADDRESS,addr,bytes) 
		return vals

	def powerUp(self,x):
		self.I2C.writeBulk(self.ADDRESS,[0x6B,0])

	def setGyroRange(self,rs):
		self.GR=self.params['setGyroRange'].index(rs)
		self.I2C.writeBulk(self.ADDRESS,[self.GYRO_CONFIG,self.GR<<3])
		
	def setAccelRange(self,rs):
		self.AR=self.params['setAccelRange'].index(rs)
		self.I2C.writeBulk(self.ADDRESS,[self.ACCEL_CONFIG,self.AR<<3])

	def getRaw(self):
		'''
		This method must be defined if you want GUIs to use this class to generate 
		plots on the fly.
		It must return a set of different values read from the sensor. such as X,Y,Z acceleration.
		The length of this list must not change, and must be defined in the variable NUMPLOTS.
		
		GUIs will generate as many plots, and the data returned from this method will be appended appropriately
		'''
		vals=self.getVals(0x3B,14)
		if vals:
			if len(vals)==14:
				raw=[0]*7
				for a in range(3):raw[a] = 1.*int16(vals[a*2]<<8|vals[a*2+1])/self.ACCEL_SCALING[self.AR]
				for a in range(4,7):raw[a] = 1.*int16(vals[a*2]<<8|vals[a*2+1])/self.GYRO_SCALING[self.GR]
				raw[3] = int16(vals[6]<<8|vals[7])/340. + 36.53
				return raw

			else:
				return False
		else:
			return False

	def getAccel(self):
		'''
		Return a list of 3 values for acceleration vector
		
		'''
		vals=self.getVals(0x3B,6)
		ax=int16(vals[0]<<8|vals[1])
		ay=int16(vals[2]<<8|vals[3])
		az=int16(vals[4]<<8|vals[5])
		return [ax/65535.,ay/65535.,az/65535.]

	def getTemp(self):
		'''
		Return temperature
		'''
		vals=self.getVals(0x41,6)
		t=int16(vals[0]<<8|vals[1])
		return t/65535.

	def getGyro(self):
		'''
		Return a list of 3 values for angular velocity vector
		
		'''
		vals=self.getVals(0x43,6)
		ax=int16(vals[0]<<8|vals[1])
		ay=int16(vals[2]<<8|vals[3])
		az=int16(vals[4]<<8|vals[5])
		return [ax/65535.,ay/65535.,az/65535.]

	def getMag(self):
		'''
		Return a list of 3 values for magnetic field vector
		
		'''
		vals=self.I2C.readBulk(self.AK8963_ADDRESS,0x03,7) #6+1 . 1(ST2) should not have bit 4 (0x8) true. It's ideally 16 . overflow bit
		if vals is not None:
			ax,ay,az = struct.unpack('<hhh',bytes(vals[:6]))
		if not vals[6]&0x08:return [ax,ay,az]
		else: return None


	def WhoAmI(self):
		'''
		Returns the ID . 
		It is 71 for MPU9250 . 
		'''
		v = self.I2C.readBulk(self.ADDRESS,0x75,1)[0]
		if v not in [0x71,0x73]:return 'Error %s'%hex(v)

		
		if v==0x73:return 'MPU9255 %s'%hex(v)
		elif v==0x71:return 'MPU9250 %s'%hex(v)
		

	def WhoAmI_AK8963(self):
		'''
		Returns the ID fo magnetometer AK8963 if found. 
		It should be 0x48. 
		'''
		self.initMagnetometer()
		v= self.I2C.readBulk(self.AK8963_ADDRESS,0,1) [0]
		if v==0x48:return 'AK8963 at %s'%hex(v)
		else: return 'AK8963 not found. returned :%s'%hex(v)

	def initMagnetometer(self):
		'''
		For MPU925x with integrated magnetometer.
		It's called a 10 DoF sensor, but technically speaking , 
		the 3-axis Accel , 3-Axis Gyro, temperature sensor are integrated in one IC, and the 3-axis magnetometer is implemented in a 
		separate IC which can be accessed via an I2C passthrough.
		
		Therefore , in order to detect the magnetometer via an I2C scan, the passthrough must first be enabled on IC#1 (Accel,gyro,temp)
		'''
		self.I2C.writeBulk(self.ADDRESS,[self.INT_PIN_CFG,0x22]) #I2C passthrough
		self.I2C.writeBulk(self.AK8963_ADDRESS,[self.AK8963_CNTL,0]) #power down mag
		self.I2C.writeBulk(self.AK8963_ADDRESS,[self.AK8963_CNTL,(1<<4)|6]) #mode   (0=14bits,1=16bits) <<4 | (2=8Hz , 6=100Hz)


		
if __name__ == "__main__":
	from expeyes import eyes17
	I= eyes17.open()
	A = connect(I.I2C) 
	t,x,y,z = I.I2C.capture(A.ADDRESS,0x43,6,5000,1000,'int') 
	#print (t,x,y,z)
	from matplotlib import pyplot as plt
	plot(t,x)
	plot(t,y)
	plot(t,z)
	show()



from __future__ import print_function
import numpy as np

gains = [1, 2, 4, 5, 8, 10, 16, 32, 1 / 11.]

# -----------------------Classes for input sources----------------------
# Variables , and functions
# allAnalogChannels : A list of available input channels that are used to populate drop down menus in apps like the oscilloscope
#                     Its use can be avoided if the inputRanges dict is migrated to OrderedDict
# bipolars : bipolar input channels. Accept both positive and negative input voltages
# inputRanges : a dictionary with channel_name as key , and a tuple as value. The tuple's first part is the voltage for a raw ADC
#               reading of 0 , and second value is for a raw ADC reading of 4095 in 12-bit mode. These are used for scaling in the
#               absence of calibration data. Since A1, A2 are fed through an inverting op-amp in the hardware, raw ADC 0 = 16.5V

allAnalogChannels = ['A1', 'A2', 'A3', 'MIC', 'IN1', 'SEN', 'AN8', 'CCS']

bipolars = ['A1', 'A2', 'A3', 'MIC']
inputRanges = {'A1': [16.5, -16.5],  # Specify inverted channels explicitly by reversing range!!
               'A2': [16.5, -16.5],
               'A3': [-3.3, 3.3],  # external gain control analog input
               'MIC': [-3.3, 3.3],  # connected to MIC amplifier
               'IN1': [0, 3.3],
               'SEN': [0, 3.3],
               'AN8': [0, 3.3],
               'CCS': [0, 3.3],
               }

picADCMultiplex = {'A1': 3, 'A2': 0, 'A3': 1, 'MIC': 2, 'IN1': 5, 'CCS': 6, 'SEN': 7, 'AN8': 8}
gainPGAs = {'A1': 1, 'A2': 2}


class analogInputSource:
    gain_values = gains
    gainEnabled = False
    gain = None
    gainPGA = None
    inverted = False
    inversion = 1.
    calPoly10 = np.poly1d([0, 3.3 / 1023, 0.])
    calPoly12 = np.poly1d([0, 3.3 / 4095, 0.])
    calibrationReady = False
    defaultOffsetCode = 0

    def __init__(self, name, **args):
        self.name = name  # The generic name of the input. like 'A1', 'IN1' etc
        self.CHOSA = picADCMultiplex[self.name]
        self.polynomials = {}
        self.calibrationCorrection = {}  # Percentage error of calibration coefficient at extremum
        self.calibrationError = False  # Will be set to True if any channel exceeds 30% error

        self.R = inputRanges[name]

        if self.R[1] - self.R[0] < 0:
            self.inverted = True
            self.inversion = -1

        self.scaling = 1.
        if name in gainPGAs:
            self.gainEnabled = True
            self.gainPGA = gainPGAs[name]
            self.gain = 0  # This is not the gain factor. use self.gain_values[self.gain] to find that.

        self.gain = 0
        self.regenerateCalibration()

    def setGain(self, g):
        '''
		specify gain 1,2,4,5,8,10,16,32
		used for correctly scaling ADC codes
		'''
        if not self.gainEnabled:
            print('Analog gain is not available on', self.name)
            return False
        self.gain = self.gain_values.index(g)
        self.regenerateCalibration()

    def __setGain__(self, g):
        '''
		specify gain index 0,1,2,3,4,5,6,7
		used for correctly scaling ADC codes
		'''
        if not self.gainEnabled:
            print('Analog gain is not available on', self.name)
            return False
        self.gain = g
        self.regenerateCalibration()

    def inRange(self, val):
        v = self.voltToCode12(val)
        return (v >= 0 and v <= 4095)

    def __conservativeInRange__(self, val, delta=100):
        v = self.voltToCode12(val)
        return (v >= 0 + delta and v <= 4095 - delta)

    def __conservativeInRangeRaw__(self, val, delta=100):
        return (val >= 0 + delta and val <= 4095 - delta)

    def __ignoreCalibration__(self):
        self.calibrationError = True
        self.calibrationReady = False

    def loadPolynomials(self, polys):
        B = self.R[1]
        A = self.R[0]
        intercept = self.R[0]

        for a in range(len(polys)):
            epoly = [float(b) for b in polys[a]]
            newPoly = np.poly1d(epoly)

            gain = self.gain_values[a]
            slope = (B - A) / gain / 4095.
            intercept = A / gain
            idealPoly = np.poly1d([0, slope, intercept])
            # print (self.name,a,newPoly(4095),idealPoly(4095))
            err = 100 * abs((newPoly(4095) - idealPoly(4095)) / idealPoly(4095))
            self.calibrationCorrection[a] = err
            if (err > 30.):  # Greater than 30% deviation from ideal
                self.calibrationError = True
                self.polynomials[a] = idealPoly
                print('Calibration invalid for %s at Gain %dx | %.2f' % (self.name, gain, err))
            else:
                self.polynomials[a] = newPoly

    def regenerateCalibration(self):
        B = self.R[1]
        A = self.R[0]
        intercept = self.R[0]

        if self.gain != None:
            gain = self.gain_values[self.gain]
            B = B / gain
            A = A / gain

        slope = B - A
        intercept = A
        if self.calibrationReady and self.gain != 8:  # special case for 1/11. gain
            self.calPoly10 = self.__cal10__
            self.calPoly12 = self.__cal12__

            # self.voltToCode10 = np.poly1d([0,1023./slope,-1023*intercept/slope])
            # self.voltToCode12 = np.poly1d([0,4095./slope,-4095*intercept/slope])
            # Inverse of the calibration polynomials .
            self.voltToCode12 = lambda x: np.clip((self.polynomials[self.gain] - x).roots[1], 0, 4090)
            self.voltToCode10 = lambda x: np.clip(((self.polynomials[self.gain] - x).roots[1]) * 1023. / 4095, 3, 1020)

        else:
            self.calPoly10 = np.poly1d([0, slope / 1023., intercept])
            self.calPoly12 = np.poly1d([0, slope / 4095., intercept])

            self.voltToCode10 = np.poly1d([0, 1023. / slope, -1023 * intercept / slope])
            self.voltToCode12 = np.poly1d([0, 4095. / slope, -4095 * intercept / slope])

    def __cal12__(self, RAW):
        return self.polynomials[self.gain](RAW)

    def __cal10__(self, RAW):
        RAW *= 4095 / 1023.
        return self.polynomials[self.gain](RAW)


'''
for a in ['CH1']:
	x=analogInputSource(a)
	print (x.name,x.calPoly10#,calfacs[x.name][0])
	print ('CAL:',x.calPoly10(0),x.calPoly10(1023))
	x.setOffset(1.65)
	x.setGain(32)
	print (x.name,x.calPoly10#,calfacs[x.name][0])
	print ('CAL:',x.calPoly10(0),x.calPoly10(1023))
'''


# ---------------------------------------------------------------------


class analogAcquisitionChannel:
    """
    This class takes care of oscilloscope data fetched from the device.
    Each instance may be linked to a particular input.
    Since only up to two channels may be captured at a time with the vLabtool, only two instances will be required

    Each instance will be linked to a particular inputSource instance by the capture routines.
    When data is requested , it will return after applying calibration and gain details
    stored in the selected inputSource
    """

    def __init__(self, a):
        self.name = ''
        self.gain = 0
        self.channel = a
        self.channel_names = allAnalogChannels
        # REFERENCE VOLTAGE = 3.3 V
        self.calibration_ref196 = 1.  # measured reference voltage/3.3
        self.resolution = 10
        self.xaxis = np.zeros(10000)
        self.yaxis = np.zeros(10000)
        self.length = 100
        self.fetched_length = 0
        self.timebase = 1.
        self.source = analogInputSource('A1')  # use CH1 for initialization. It will be overwritten by set_params

    def reset(self):
        self.fetched_length = 0

    def fix_value(self, val):
        # val[val>1020]=np.NaN
        # val[val<2]=np.NaN
        if self.resolution == 12:
            return self.calibration_ref196 * self.source.calPoly12(val)
        else:
            return self.calibration_ref196 * self.source.calPoly10(val)

    def set_yval(self, pos, val):
        self.yaxis[pos] = self.fix_value(val)

    def set_xval(self, pos, val):
        self.xaxis[pos] = val

    def set_params(self, **keys):
        self.gain = keys.get('gain', self.gain)
        self.name = keys.get('channel', self.channel)
        self.source = keys.get('source', self.source)
        self.resolution = keys.get('resolution', self.resolution)
        l = keys.get('length', self.length)
        t = keys.get('timebase', self.timebase)
        self.fetched_length = 0
        if t != self.timebase or l != self.length:
            self.timebase = t
            self.length = l
            self.regenerate_xaxis()

    def regenerate_xaxis(self):
        for a in range(int(self.length)):
            self.xaxis[a] = self.timebase * a

    def get_xaxis(self):
        return self.xaxis[:self.length]

    def get_yaxis(self):
        return self.yaxis[:self.length]

    def get_fetched_xaxis(self):
        return self.xaxis[:self.fetched_length]

    def get_fetched_yaxis(self):
        return self.yaxis[:self.fetched_length]



import sys
from time import *


# commands
LCD_CLEARDISPLAY = 0x01
LCD_RETURNHOME = 0x02
LCD_ENTRYMODESET = 0x04
LCD_DISPLAYCONTROL = 0x08
LCD_CURSORSHIFT = 0x10
LCD_FUNCTIONSET = 0x20
LCD_SETCGRAMADDR = 0x40
LCD_SETDDRAMADDR = 0x80

# flags for display entry mode
LCD_ENTRYRIGHT = 0x00
LCD_ENTRYLEFT = 0x02
LCD_ENTRYSHIFTINCREMENT = 0x01
LCD_ENTRYSHIFTDECREMENT = 0x00

# flags for display on/off control
LCD_DISPLAYON = 0x04
LCD_DISPLAYOFF = 0x00
LCD_CURSORON = 0x02
LCD_CURSOROFF = 0x00
LCD_BLINKON = 0x01
LCD_BLINKOFF = 0x00

# flags for display/cursor shift
LCD_DISPLAYMOVE = 0x08
LCD_CURSORMOVE = 0x00
LCD_MOVERIGHT = 0x04
LCD_MOVELEFT = 0x00

# flags for function set
LCD_8BITMODE = 0x10
LCD_4BITMODE = 0x00
LCD_2LINE = 0x08
LCD_1LINE = 0x00
LCD_5x10DOTS = 0x04
LCD_5x8DOTS = 0x00

# flags for backlight control
LCD_BACKLIGHT = 0x08
LCD_NOBACKLIGHT = 0x00

En = 0b00000100 # Enable bit
Rw = 0b00000010 # Read/Write bit
Rs = 0b00000001 # Register select bit


class i2c_device:
	def __init__(self, I2C, addr):
		self.I2C = I2C
		self.I2CWriteBulk = I2C.writeBulk
		self.addr = addr

	# Write a single command
	def write_cmd(self, cmd):
		self.I2CWriteBulk(self.addr, [cmd])
		sleep(0.0001)

	# Write a command and argument
	def write_cmd_arg(self, cmd, data):
		self.I2CWriteBulk(self.addr, [cmd, data])
		sleep(0.0001)	  

class lcd:
	#initializes objects and lcd
	def __init__(self, I2C, ADDRESS):
		self.lcd_device = i2c_device(I2C, ADDRESS)
		self.lcd_write(0x03)
		self.lcd_write(0x03)
		self.lcd_write(0x03)
		self.lcd_write(0x02)

		self.lcd_write(LCD_FUNCTIONSET | LCD_2LINE | LCD_5x8DOTS | LCD_4BITMODE)
		self.lcd_write(LCD_DISPLAYCONTROL | LCD_DISPLAYON)
		self.lcd_write(LCD_CLEARDISPLAY)
		self.lcd_write(LCD_ENTRYMODESET | LCD_ENTRYLEFT)
		sleep(0.2)

	# clocks EN to latch command
	def lcd_strobe(self, data):
		self.lcd_device.write_cmd(data | En | LCD_BACKLIGHT)
		sleep(.0005)
		self.lcd_device.write_cmd(((data & ~En) | LCD_BACKLIGHT))
		sleep(.0001)

	def lcd_write_four_bits(self, data):
		self.lcd_device.write_cmd(data | LCD_BACKLIGHT)
		self.lcd_strobe(data)

	# write a command to lcd
	def lcd_write(self, cmd, mode=0):
		self.lcd_write_four_bits(mode | (cmd & 0xF0))
		self.lcd_write_four_bits(mode | ((cmd << 4) & 0xF0))
	  
	#turn on/off the lcd backlight
	def lcd_backlight(self, state):
		if state in ("on","On","ON"):
			self.lcd_device.write_cmd(LCD_BACKLIGHT)
		elif state in ("off","Off","OFF"):
			self.lcd_device.write_cmd(LCD_NOBACKLIGHT)
		else:
			print("Unknown State!")

	# put string function
	def lcd_display_string(self, string, line):
		if line == 1:
			self.lcd_write(0x80)
		if line == 2:
			self.lcd_write(0xC0)
		if line == 3:
			self.lcd_write(0x94)
		if line == 4:
			self.lcd_write(0xD4)

		for char in string:
			self.lcd_write(ord(char), Rs)

	# clear lcd and set to home
	def lcd_clear(self):
		self.lcd_write(LCD_CLEARDISPLAY)
		self.lcd_write(LCD_RETURNHOME)



from __future__  import print_function
from numpy import int16
def connect(route,**args):
	return BRIDGE(route,**args)


class BRIDGE():
	POWER_ON =0x01
	RESET =0x07
	RES_1000mLx  =0x10
	RES_500mLx  =0x11
	RES_4000mLx  =0x13
	
	gain_choices=[RES_500mLx,RES_1000mLx,RES_4000mLx]
	gain_literal_choices=['500mLx','1000mLx','4000mLx']
	gain = 0
	scaling=[2,1,2]
	
	#--------------Parameters--------------------
	#This must be defined in order to let GUIs automatically create menus
	#for changing various options of this sensor
	#It's a dictionary of the string representations of functions matched with an array
	#of options that each one can accept
	params={'init':['Now'],
	'setRange':gain_literal_choices,
	}
	
	NUMPLOTS=1	
	PLOTNAMES = ['Lux']
	ADDRESS = 0x23
	name = 'Luminosity'
	def __init__(self,I2C,**args):
		self.I2C=I2C
		self.ADDRESS = args.get('address',0x23)
		self.init('')

	def init(self,dummy_variable_to_circumvent_framework_limitation):  # I know how to fix this now. remind me.
		self.I2C.writeBulk(self.ADDRESS,[self.RES_500mLx]) 

	def setRange(self,g):
		self.gain = self.gain_literal_choices.index(g)
		self.I2C.writeBulk(self.ADDRESS,[self.gain_choices[self.gain]]) 

	def getVals(self,numbytes):
		vals = self.I2C.simpleRead(self.ADDRESS,numbytes) 
		return vals
	
	def getRaw(self):
		vals=self.getVals(2)
		if vals:
			if len(vals)==2:
				return [(vals[0]<<8|vals[1])/1.2]  #/self.scaling[self.gain]
			else:
				return False
		else:
			return False
		

if __name__ == "__main__":
	from expeyes import eyes17
	I= eyes17.open()
	A = connect(I.I2C)
	print (A.getRaw())



from __future__ import print_function
from numpy import int16
def connect(route,**args):
	return MLX90614(route,**args)

class MLX90614():
	NUMPLOTS=1
	PLOTNAMES=['Temp']
	ADDRESS = 0x5A
	name = 'PIR temperature'
	def __init__(self,I2C,**args):
		self.I2C=I2C
		self.ADDRESS = args.get('address',self.ADDRESS)
		self.OBJADDR=0x07
		self.AMBADDR=0x06

		self.source=self.OBJADDR
		
		self.name = 'Passive IR temperature sensor'
		self.params={'readReg':range(0x20),
		'select_source':['object temperature','ambient temperature']}
		
		try:
			print ('switching baud to 100k')
			self.I2C.configI2C(100e3)
		except:
			print ('FAILED TO CHANGE BAUD RATE')


	def select_source(self,source):
		if source=='object temperature':
			self.source = self.OBJADDR
		elif source=='ambient temperature':
			self.source = self.AMBADDR


	def readReg(self,addr):
		x= self.getVals(addr,2)
		print (hex(addr),hex(x[0]|(x[1]<<8)))


	def getVals(self,addr,bytes):
		vals = self.I2C.readBulk(self.ADDRESS,addr,bytes) 
		return vals

	def getRaw(self):
		vals=self.getVals(self.source,3)
		if vals:
			if len(vals)==3:
				return [((((vals[1]&0x007f)<<8)+vals[0])*0.02)-0.01 - 273.15]
			else:
				return False
		else:
			return False

	def getObjectTemperature(self):
		self.source = self.OBJADDR
		val=self.getRaw()
		if val:
			return val[0]
		else:
			return False

	def getAmbientTemperature(self):
		self.source = self.AMBADDR
		val=self.getRaw()
		if val:
			return val[0]
		else:
			return False



from numpy import int16
def connect(route,**args):
	return HMC5883L(route,**args)


class HMC5883L():
	CONFA=0x00
	CONFB=0x01
	MODE=0x02
	STATUS=0x09
	
	#--------CONFA register bits. 0x00-----------
	samplesToAverage=0
	samplesToAverage_choices=[1,2,4,8]
	
	dataOutputRate=6
	dataOutputRate_choices=[0.75,1.5,3,7.5,15,30,75]
	
	measurementConf=0
	
	#--------CONFB register bits. 0x01-----------
	gainValue = 7 #least sensitive
	gain_choices = [8,7,6,5,4,3,2,1]
	scaling=[1370.,1090.,820.,660.,440.,390.,330.,230.]
	
	#--------------Parameters--------------------
	#This must be defined in order to let GUIs automatically create menus
	#for changing various options of this sensor
	#It's a dictionary of the string representations of functions matched with an array
	#of options that each one can accept
	params={	'init':['Now'],
	'setSamplesToAverage':samplesToAverage_choices,
	'setDataOutputRate':dataOutputRate_choices,
	'setGain':gain_choices,
	}
	ADDRESS = 0x1E
	name = 'Magnetometer'
	NUMPLOTS=3	
	PLOTNAMES = ['Bx','By','Bz']
	def __init__(self,I2C,**args):
		self.I2C=I2C
		self.ADDRESS = args.get('address',self.ADDRESS)
		self.name = 'Magnetometer'
		'''
		try:
			print 'switching baud to 400k'
			self.I2C.configI2C(400e3)
		except:
			print 'FAILED TO CHANGE BAUD RATE'
		'''
		self.init('')

	def init(self,dummy_variable_to_circumvent_framework_limitation):  # I know how to fix this now. remind me.
		self.__writeCONFA__()
		self.__writeCONFB__()
		self.I2C.writeBulk(self.ADDRESS,[self.MODE,0]) #enable continuous measurement mode

	def __writeCONFB__(self):
		self.I2C.writeBulk(self.ADDRESS,[self.CONFB,self.gainValue<<5]) #set gain

	def __writeCONFA__(self):
		self.I2C.writeBulk(self.ADDRESS,[self.CONFA,(self.dataOutputRate<<2)|(self.samplesToAverage<<5)|(self.measurementConf)])
		
	def setSamplesToAverage(self,num):
		self.samplesToAverage=self.samplesToAverage_choices.index(num)
		self.__writeCONFA__()
	
	def setDataOutputRate(self,rate):
		self.dataOutputRate=self.dataOutputRate_choices.index(rate)
		self.__writeCONFA__()
	
	def setGain(self,gain):
		self.gainValue = self.gain_choices.index(gain)
		self.__writeCONFB__()
	
	def getVals(self,addr,bytes):
		vals = self.I2C.readBulk(self.ADDRESS,addr,bytes) 
		return vals
	
	def getRaw(self):
		vals=self.getVals(0x03,6)
		if vals:
			if len(vals)==6:
				return [int16(vals[a*2]<<8|vals[a*2+1])/self.scaling[self.gainValue] for a in range(3)]
			else:
				return False
		else:
			return False
		

if __name__ == "__main__":
	from expeyes import eyes17
	I= eyes17.open()
	I.set_sine1(.5)
	A = connect(I.I2C) 
	A.setGain(2)
	t,x,y,z = I.I2C.capture(A.ADDRESS,0x03,6,400,10000,'int') 
	#print (t,x,y,z)
	from matplotlib import pyplot as plt
	plot(t,x)
	plot(t,y)
	plot(t,z)
	show()



# -*- coding: utf-8; mode: python; indent-tabs-mode: t; tab-width:4 -*-
import inspect

from . import HMC5883L, MPU6050, MLX90614, BMP180, TSL2561, SHT21, BH1750, ADS1115, QMC5883L, BMP280, VL53L01X

supported = {
    0x48: ADS1115,  # 16-bit ADC
    0x23: BH1750,  # Luminosity
    0x77: BMP180,  # Pressure, Temperature
    0x78: BMP280,  # Pressure, Temperature, altitude
    0x29: VL53L01X,  # distance. LIDAR #41 . address
    0x68: MPU6050,  # 3-axis gyro,3-axis accel,temperature
    0x1E: HMC5883L,  # 3-axis magnetometer
    13: QMC5883L,  # 3-axis magnetometer
    0x5A: MLX90614,  # Passive IR temperature sensor
    0x39: TSL2561,  # Luminosity
    0x40: SHT21,  # Temperature, Humidity
}

# auto generated map of names to classes
nameMap = {}
for a in supported:
    nameMap[supported[a].__name__.split('.')[-1]] = (supported[a])
